package org.lazywizard.console.ui

/**
 * This is a completely optional container for user interface elements. All it does is direct calls to [advance()]
 * [UIElement.advance] and [render()][UIElement.render] to each [UIElement] it contains, simplifying the code when
 * working with multiple user interface elements.
 */
open class UI {
    val elements = HashSet<UIElement>()

    fun addElement(element: UIElement) {
        elements.add(element)
    }

    fun removeElement(element: UIElement) {
        elements.remove(element)
    }

    fun advance(amount: Float) {
        for (element in elements)
            element.advance(amount)
    }

    fun render() {
        for (element in elements)
            element.render()
    }
}

interface UIElement {
    fun advance(amount: Float)
    fun render()
}

class ScrollableTextArea : UIElement {
    override fun advance(amount: Float) {}
    override fun render() {}
}
